# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.api import monitored_resource_pb2  # type: ignore
from google.protobuf import duration_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "UptimeCheckRegion",
        "GroupResourceType",
        "InternalChecker",
        "SyntheticMonitorTarget",
        "UptimeCheckConfig",
        "UptimeCheckIp",
    },
)


class UptimeCheckRegion(proto.Enum):
    r"""The regions from which an Uptime check can be run.

    Values:
        REGION_UNSPECIFIED (0):
            Default value if no region is specified. Will
            result in Uptime checks running from all
            regions.
        USA (1):
            Allows checks to run from locations within
            the United States of America.
        EUROPE (2):
            Allows checks to run from locations within
            the continent of Europe.
        SOUTH_AMERICA (3):
            Allows checks to run from locations within
            the continent of South America.
        ASIA_PACIFIC (4):
            Allows checks to run from locations within
            the Asia Pacific area (ex: Singapore).
        USA_OREGON (5):
            Allows checks to run from locations within
            the western United States of America
        USA_IOWA (6):
            Allows checks to run from locations within
            the central United States of America
        USA_VIRGINIA (7):
            Allows checks to run from locations within
            the eastern United States of America
    """
    REGION_UNSPECIFIED = 0
    USA = 1
    EUROPE = 2
    SOUTH_AMERICA = 3
    ASIA_PACIFIC = 4
    USA_OREGON = 5
    USA_IOWA = 6
    USA_VIRGINIA = 7


class GroupResourceType(proto.Enum):
    r"""The supported resource types that can be used as values of
    ``group_resource.resource_type``. ``INSTANCE`` includes
    ``gce_instance`` and ``aws_ec2_instance`` resource types. The
    resource types ``gae_app`` and ``uptime_url`` are not valid here
    because group checks on App Engine modules and URLs are not allowed.

    Values:
        RESOURCE_TYPE_UNSPECIFIED (0):
            Default value (not valid).
        INSTANCE (1):
            A group of instances from Google Cloud
            Platform (GCP) or Amazon Web Services (AWS).
        AWS_ELB_LOAD_BALANCER (2):
            A group of Amazon ELB load balancers.
    """
    RESOURCE_TYPE_UNSPECIFIED = 0
    INSTANCE = 1
    AWS_ELB_LOAD_BALANCER = 2


class InternalChecker(proto.Message):
    r"""An internal checker allows Uptime checks to run on
    private/internal GCP resources.

    Attributes:
        name (str):
            A unique resource name for this InternalChecker. The format
            is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/internalCheckers/[INTERNAL_CHECKER_ID]

            ``[PROJECT_ID_OR_NUMBER]`` is the Cloud Monitoring Metrics
            Scope project for the Uptime check config associated with
            the internal checker.
        display_name (str):
            The checker's human-readable name. The
            display name should be unique within a Cloud
            Monitoring Metrics Scope in order to make it
            easier to identify; however, uniqueness is not
            enforced.
        network (str):
            The `GCP VPC
            network <https://cloud.google.com/vpc/docs/vpc>`__ where the
            internal resource lives (ex: "default").
        gcp_zone (str):
            The GCP zone the Uptime check should egress from. Only
            respected for internal Uptime checks, where internal_network
            is specified.
        peer_project_id (str):
            The GCP project ID where the internal checker
            lives. Not necessary the same as the Metrics
            Scope project.
        state (google.cloud.monitoring_v3.types.InternalChecker.State):
            The current operational state of the internal
            checker.
    """

    class State(proto.Enum):
        r"""Operational states for an internal checker.

        Values:
            UNSPECIFIED (0):
                An internal checker should never be in the
                unspecified state.
            CREATING (1):
                The checker is being created, provisioned, and configured. A
                checker in this state can be returned by
                ``ListInternalCheckers`` or ``GetInternalChecker``, as well
                as by examining the `long running
                Operation <https://cloud.google.com/apis/design/design_patterns#long_running_operations>`__
                that created it.
            RUNNING (2):
                The checker is running and available for use. A checker in
                this state can be returned by ``ListInternalCheckers`` or
                ``GetInternalChecker`` as well as by examining the `long
                running
                Operation <https://cloud.google.com/apis/design/design_patterns#long_running_operations>`__
                that created it. If a checker is being torn down, it is
                neither visible nor usable, so there is no "deleting" or
                "down" state.
        """
        UNSPECIFIED = 0
        CREATING = 1
        RUNNING = 2

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    network: str = proto.Field(
        proto.STRING,
        number=3,
    )
    gcp_zone: str = proto.Field(
        proto.STRING,
        number=4,
    )
    peer_project_id: str = proto.Field(
        proto.STRING,
        number=6,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=7,
        enum=State,
    )


class SyntheticMonitorTarget(proto.Message):
    r"""Describes a Synthetic Monitor to be invoked by Uptime.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        cloud_function_v2 (google.cloud.monitoring_v3.types.SyntheticMonitorTarget.CloudFunctionV2Target):
            Target a Synthetic Monitor GCFv2 instance.

            This field is a member of `oneof`_ ``target``.
    """

    class CloudFunctionV2Target(proto.Message):
        r"""A Synthetic Monitor deployed to a Cloud Functions V2
        instance.

        Attributes:
            name (str):
                Required. Fully qualified GCFv2 resource name i.e.
                ``projects/{project}/locations/{location}/functions/{function}``
                Required.
            cloud_run_revision (google.api.monitored_resource_pb2.MonitoredResource):
                Output only. The ``cloud_run_revision`` Monitored Resource
                associated with the GCFv2. The Synthetic Monitor execution
                results (metrics, logs, and spans) are reported against this
                Monitored Resource. This field is output only.
        """

        name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        cloud_run_revision: monitored_resource_pb2.MonitoredResource = proto.Field(
            proto.MESSAGE,
            number=2,
            message=monitored_resource_pb2.MonitoredResource,
        )

    cloud_function_v2: CloudFunctionV2Target = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="target",
        message=CloudFunctionV2Target,
    )


class UptimeCheckConfig(proto.Message):
    r"""This message configures which resources and services to
    monitor for availability.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Identifier. A unique resource name for this Uptime check
            configuration. The format is:

            ::

                 projects/[PROJECT_ID_OR_NUMBER]/uptimeCheckConfigs/[UPTIME_CHECK_ID]

            ``[PROJECT_ID_OR_NUMBER]`` is the Workspace host project
            associated with the Uptime check.

            This field should be omitted when creating the Uptime check
            configuration; on create, the resource name is assigned by
            the server and included in the response.
        display_name (str):
            A human-friendly name for the Uptime check
            configuration. The display name should be unique
            within a Cloud Monitoring Workspace in order to
            make it easier to identify; however, uniqueness
            is not enforced. Required.
        monitored_resource (google.api.monitored_resource_pb2.MonitoredResource):
            The `monitored
            resource <https://cloud.google.com/monitoring/api/resources>`__
            associated with the configuration. The following monitored
            resource types are valid for this field: ``uptime_url``,
            ``gce_instance``, ``gae_app``, ``aws_ec2_instance``,
            ``aws_elb_load_balancer`` ``k8s_service``
            ``servicedirectory_service`` ``cloud_run_revision``

            This field is a member of `oneof`_ ``resource``.
        resource_group (google.cloud.monitoring_v3.types.UptimeCheckConfig.ResourceGroup):
            The group resource associated with the
            configuration.

            This field is a member of `oneof`_ ``resource``.
        synthetic_monitor (google.cloud.monitoring_v3.types.SyntheticMonitorTarget):
            Specifies a Synthetic Monitor to invoke.

            This field is a member of `oneof`_ ``resource``.
        http_check (google.cloud.monitoring_v3.types.UptimeCheckConfig.HttpCheck):
            Contains information needed to make an HTTP
            or HTTPS check.

            This field is a member of `oneof`_ ``check_request_type``.
        tcp_check (google.cloud.monitoring_v3.types.UptimeCheckConfig.TcpCheck):
            Contains information needed to make a TCP
            check.

            This field is a member of `oneof`_ ``check_request_type``.
        period (google.protobuf.duration_pb2.Duration):
            How often, in seconds, the Uptime check is performed.
            Currently, the only supported values are ``60s`` (1 minute),
            ``300s`` (5 minutes), ``600s`` (10 minutes), and ``900s``
            (15 minutes). Optional, defaults to ``60s``.
        timeout (google.protobuf.duration_pb2.Duration):
            The maximum amount of time to wait for the
            request to complete (must be between 1 and 60
            seconds). Required.
        content_matchers (MutableSequence[google.cloud.monitoring_v3.types.UptimeCheckConfig.ContentMatcher]):
            The content that is expected to appear in the data returned
            by the target server against which the check is run.
            Currently, only the first entry in the ``content_matchers``
            list is supported, and additional entries will be ignored.
            This field is optional and should only be specified if a
            content match is required as part of the/ Uptime check.
        checker_type (google.cloud.monitoring_v3.types.UptimeCheckConfig.CheckerType):
            The type of checkers to use to execute the
            Uptime check.
        selected_regions (MutableSequence[google.cloud.monitoring_v3.types.UptimeCheckRegion]):
            The list of regions from which the check will
            be run. Some regions contain one location, and
            others contain more than one. If this field is
            specified, enough regions must be provided to
            include a minimum of 3 locations.  Not
            specifying this field will result in Uptime
            checks running from all available regions.
        is_internal (bool):
            If this is ``true``, then checks are made only from the
            'internal_checkers'. If it is ``false``, then checks are
            made only from the 'selected_regions'. It is an error to
            provide 'selected_regions' when is_internal is ``true``, or
            to provide 'internal_checkers' when is_internal is
            ``false``.
        internal_checkers (MutableSequence[google.cloud.monitoring_v3.types.InternalChecker]):
            The internal checkers that this check will egress from. If
            ``is_internal`` is ``true`` and this list is empty, the
            check will egress from all the InternalCheckers configured
            for the project that owns this ``UptimeCheckConfig``.
        user_labels (MutableMapping[str, str]):
            User-supplied key/value data to be used for organizing and
            identifying the ``UptimeCheckConfig`` objects.

            The field can contain up to 64 entries. Each key and value
            is limited to 63 Unicode characters or 128 bytes, whichever
            is smaller. Labels and values can contain only lowercase
            letters, numerals, underscores, and dashes. Keys must begin
            with a letter.
    """

    class CheckerType(proto.Enum):
        r"""What kind of checkers are available to be used by the check.

        Values:
            CHECKER_TYPE_UNSPECIFIED (0):
                The default checker type. Currently converted to
                ``STATIC_IP_CHECKERS`` on creation, the default conversion
                behavior may change in the future.
            STATIC_IP_CHECKERS (1):
                ``STATIC_IP_CHECKERS`` are used for uptime checks that
                perform egress across the public internet.
                ``STATIC_IP_CHECKERS`` use the static IP addresses returned
                by ``ListUptimeCheckIps``.
            VPC_CHECKERS (3):
                ``VPC_CHECKERS`` are used for uptime checks that perform
                egress using Service Directory and private network access.
                When using ``VPC_CHECKERS``, the monitored resource type
                must be ``servicedirectory_service``.
        """
        CHECKER_TYPE_UNSPECIFIED = 0
        STATIC_IP_CHECKERS = 1
        VPC_CHECKERS = 3

    class ResourceGroup(proto.Message):
        r"""The resource submessage for group checks. It can be used
        instead of a monitored resource, when multiple resources are
        being monitored.

        Attributes:
            group_id (str):
                The group of resources being monitored. Should be only the
                ``[GROUP_ID]``, and not the full-path
                ``projects/[PROJECT_ID_OR_NUMBER]/groups/[GROUP_ID]``.
            resource_type (google.cloud.monitoring_v3.types.GroupResourceType):
                The resource type of the group members.
        """

        group_id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        resource_type: "GroupResourceType" = proto.Field(
            proto.ENUM,
            number=2,
            enum="GroupResourceType",
        )

    class PingConfig(proto.Message):
        r"""Information involved in sending ICMP pings alongside public
        HTTP/TCP checks. For HTTP, the pings are performed for each part
        of the redirect chain.

        Attributes:
            pings_count (int):
                Number of ICMP pings. A maximum of 3 ICMP
                pings is currently supported.
        """

        pings_count: int = proto.Field(
            proto.INT32,
            number=1,
        )

    class HttpCheck(proto.Message):
        r"""Information involved in an HTTP/HTTPS Uptime check request.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            request_method (google.cloud.monitoring_v3.types.UptimeCheckConfig.HttpCheck.RequestMethod):
                The HTTP request method to use for the check. If set to
                ``METHOD_UNSPECIFIED`` then ``request_method`` defaults to
                ``GET``.
            use_ssl (bool):
                If ``true``, use HTTPS instead of HTTP to run the check.
            path (str):
                Optional (defaults to "/"). The path to the page against
                which to run the check. Will be combined with the ``host``
                (specified within the ``monitored_resource``) and ``port``
                to construct the full URL. If the provided path does not
                begin with "/", a "/" will be prepended automatically.
            port (int):
                Optional (defaults to 80 when ``use_ssl`` is ``false``, and
                443 when ``use_ssl`` is ``true``). The TCP port on the HTTP
                server against which to run the check. Will be combined with
                host (specified within the ``monitored_resource``) and
                ``path`` to construct the full URL.
            auth_info (google.cloud.monitoring_v3.types.UptimeCheckConfig.HttpCheck.BasicAuthentication):
                The authentication information. Optional when creating an
                HTTP check; defaults to empty. Do not set both
                ``auth_method`` and ``auth_info``.
            mask_headers (bool):
                Boolean specifying whether to encrypt the header
                information. Encryption should be specified for any headers
                related to authentication that you do not wish to be seen
                when retrieving the configuration. The server will be
                responsible for encrypting the headers. On Get/List calls,
                if ``mask_headers`` is set to ``true`` then the headers will
                be obscured with ``******.``
            headers (MutableMapping[str, str]):
                The list of headers to send as part of the
                Uptime check request. If two headers have the
                same key and different values, they should be
                entered as a single header, with the value being
                a comma-separated list of all the desired values
                as described at
                https://www.w3.org/Protocols/rfc2616/rfc2616.txt
                (page 31). Entering two separate headers with
                the same key in a Create call will cause the
                first to be overwritten by the second. The
                maximum number of headers allowed is 100.
            content_type (google.cloud.monitoring_v3.types.UptimeCheckConfig.HttpCheck.ContentType):
                The content type header to use for the check. The following
                configurations result in errors:

                1. Content type is specified in both the ``headers`` field
                   and the ``content_type`` field.
                2. Request method is ``GET`` and ``content_type`` is not
                   ``TYPE_UNSPECIFIED``
                3. Request method is ``POST`` and ``content_type`` is
                   ``TYPE_UNSPECIFIED``.
                4. Request method is ``POST`` and a "Content-Type" header is
                   provided via ``headers`` field. The ``content_type``
                   field should be used instead.
            custom_content_type (str):
                A user provided content type header to use for the check.
                The invalid configurations outlined in the ``content_type``
                field apply to ``custom_content_type``, as well as the
                following:

                1. ``content_type`` is ``URL_ENCODED`` and
                   ``custom_content_type`` is set.
                2. ``content_type`` is ``USER_PROVIDED`` and
                   ``custom_content_type`` is not set.
            validate_ssl (bool):
                Boolean specifying whether to include SSL certificate
                validation as a part of the Uptime check. Only applies to
                checks where ``monitored_resource`` is set to
                ``uptime_url``. If ``use_ssl`` is ``false``, setting
                ``validate_ssl`` to ``true`` has no effect.
            body (bytes):
                The request body associated with the HTTP POST request. If
                ``content_type`` is ``URL_ENCODED``, the body passed in must
                be URL-encoded. Users can provide a ``Content-Length``
                header via the ``headers`` field or the API will do so. If
                the ``request_method`` is ``GET`` and ``body`` is not empty,
                the API will return an error. The maximum byte size is 1
                megabyte.

                Note: If client libraries aren't used (which performs the
                conversion automatically) base64 encode your ``body`` data
                since the field is of ``bytes`` type.
            accepted_response_status_codes (MutableSequence[google.cloud.monitoring_v3.types.UptimeCheckConfig.HttpCheck.ResponseStatusCode]):
                If present, the check will only pass if the
                HTTP response status code is in this set of
                status codes. If empty, the HTTP status code
                will only pass if the HTTP status code is
                200-299.
            ping_config (google.cloud.monitoring_v3.types.UptimeCheckConfig.PingConfig):
                Contains information needed to add pings to
                an HTTP check.
            service_agent_authentication (google.cloud.monitoring_v3.types.UptimeCheckConfig.HttpCheck.ServiceAgentAuthentication):
                If specified, Uptime will generate and attach an OIDC JWT
                token for the Monitoring service agent service account as an
                ``Authorization`` header in the HTTP request when probing.

                This field is a member of `oneof`_ ``auth_method``.
        """

        class RequestMethod(proto.Enum):
            r"""The HTTP request method options.

            Values:
                METHOD_UNSPECIFIED (0):
                    No request method specified.
                GET (1):
                    GET request.
                POST (2):
                    POST request.
            """
            METHOD_UNSPECIFIED = 0
            GET = 1
            POST = 2

        class ContentType(proto.Enum):
            r"""Header options corresponding to the content type of a HTTP
            request body.

            Values:
                TYPE_UNSPECIFIED (0):
                    No content type specified.
                URL_ENCODED (1):
                    ``body`` is in URL-encoded form. Equivalent to setting the
                    ``Content-Type`` to ``application/x-www-form-urlencoded`` in
                    the HTTP request.
                USER_PROVIDED (2):
                    ``body`` is in ``custom_content_type`` form. Equivalent to
                    setting the ``Content-Type`` to the contents of
                    ``custom_content_type`` in the HTTP request.
            """
            TYPE_UNSPECIFIED = 0
            URL_ENCODED = 1
            USER_PROVIDED = 2

        class BasicAuthentication(proto.Message):
            r"""The authentication parameters to provide to the specified resource
            or URL that requires a username and password. Currently, only `Basic
            HTTP authentication <https://tools.ietf.org/html/rfc7617>`__ is
            supported in Uptime checks.

            Attributes:
                username (str):
                    The username to use when authenticating with
                    the HTTP server.
                password (str):
                    The password to use when authenticating with
                    the HTTP server.
            """

            username: str = proto.Field(
                proto.STRING,
                number=1,
            )
            password: str = proto.Field(
                proto.STRING,
                number=2,
            )

        class ResponseStatusCode(proto.Message):
            r"""A status to accept. Either a status code class like "2xx", or
            an integer status code like "200".

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                status_value (int):
                    A status code to accept.

                    This field is a member of `oneof`_ ``status_code``.
                status_class (google.cloud.monitoring_v3.types.UptimeCheckConfig.HttpCheck.ResponseStatusCode.StatusClass):
                    A class of status codes to accept.

                    This field is a member of `oneof`_ ``status_code``.
            """

            class StatusClass(proto.Enum):
                r"""An HTTP status code class.

                Values:
                    STATUS_CLASS_UNSPECIFIED (0):
                        Default value that matches no status codes.
                    STATUS_CLASS_1XX (100):
                        The class of status codes between 100 and
                        199.
                    STATUS_CLASS_2XX (200):
                        The class of status codes between 200 and
                        299.
                    STATUS_CLASS_3XX (300):
                        The class of status codes between 300 and
                        399.
                    STATUS_CLASS_4XX (400):
                        The class of status codes between 400 and
                        499.
                    STATUS_CLASS_5XX (500):
                        The class of status codes between 500 and
                        599.
                    STATUS_CLASS_ANY (1000):
                        The class of all status codes.
                """
                STATUS_CLASS_UNSPECIFIED = 0
                STATUS_CLASS_1XX = 100
                STATUS_CLASS_2XX = 200
                STATUS_CLASS_3XX = 300
                STATUS_CLASS_4XX = 400
                STATUS_CLASS_5XX = 500
                STATUS_CLASS_ANY = 1000

            status_value: int = proto.Field(
                proto.INT32,
                number=1,
                oneof="status_code",
            )
            status_class: "UptimeCheckConfig.HttpCheck.ResponseStatusCode.StatusClass" = proto.Field(
                proto.ENUM,
                number=2,
                oneof="status_code",
                enum="UptimeCheckConfig.HttpCheck.ResponseStatusCode.StatusClass",
            )

        class ServiceAgentAuthentication(proto.Message):
            r"""Contains information needed for generating either an `OpenID Connect
            token <https://developers.google.com/identity/protocols/OpenIDConnect>`__
            or `OAuth
            token <https://developers.google.com/identity/protocols/oauth2>`__.
            The token will be generated for the Monitoring service agent service
            account.

            Attributes:
                type_ (google.cloud.monitoring_v3.types.UptimeCheckConfig.HttpCheck.ServiceAgentAuthentication.ServiceAgentAuthenticationType):
                    Type of authentication.
            """

            class ServiceAgentAuthenticationType(proto.Enum):
                r"""Type of authentication.

                Values:
                    SERVICE_AGENT_AUTHENTICATION_TYPE_UNSPECIFIED (0):
                        Default value, will result in OIDC
                        Authentication.
                    OIDC_TOKEN (1):
                        OIDC Authentication
                """
                SERVICE_AGENT_AUTHENTICATION_TYPE_UNSPECIFIED = 0
                OIDC_TOKEN = 1

            type_: "UptimeCheckConfig.HttpCheck.ServiceAgentAuthentication.ServiceAgentAuthenticationType" = proto.Field(
                proto.ENUM,
                number=1,
                enum="UptimeCheckConfig.HttpCheck.ServiceAgentAuthentication.ServiceAgentAuthenticationType",
            )

        request_method: "UptimeCheckConfig.HttpCheck.RequestMethod" = proto.Field(
            proto.ENUM,
            number=8,
            enum="UptimeCheckConfig.HttpCheck.RequestMethod",
        )
        use_ssl: bool = proto.Field(
            proto.BOOL,
            number=1,
        )
        path: str = proto.Field(
            proto.STRING,
            number=2,
        )
        port: int = proto.Field(
            proto.INT32,
            number=3,
        )
        auth_info: "UptimeCheckConfig.HttpCheck.BasicAuthentication" = proto.Field(
            proto.MESSAGE,
            number=4,
            message="UptimeCheckConfig.HttpCheck.BasicAuthentication",
        )
        mask_headers: bool = proto.Field(
            proto.BOOL,
            number=5,
        )
        headers: MutableMapping[str, str] = proto.MapField(
            proto.STRING,
            proto.STRING,
            number=6,
        )
        content_type: "UptimeCheckConfig.HttpCheck.ContentType" = proto.Field(
            proto.ENUM,
            number=9,
            enum="UptimeCheckConfig.HttpCheck.ContentType",
        )
        custom_content_type: str = proto.Field(
            proto.STRING,
            number=13,
        )
        validate_ssl: bool = proto.Field(
            proto.BOOL,
            number=7,
        )
        body: bytes = proto.Field(
            proto.BYTES,
            number=10,
        )
        accepted_response_status_codes: MutableSequence[
            "UptimeCheckConfig.HttpCheck.ResponseStatusCode"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=11,
            message="UptimeCheckConfig.HttpCheck.ResponseStatusCode",
        )
        ping_config: "UptimeCheckConfig.PingConfig" = proto.Field(
            proto.MESSAGE,
            number=12,
            message="UptimeCheckConfig.PingConfig",
        )
        service_agent_authentication: "UptimeCheckConfig.HttpCheck.ServiceAgentAuthentication" = proto.Field(
            proto.MESSAGE,
            number=14,
            oneof="auth_method",
            message="UptimeCheckConfig.HttpCheck.ServiceAgentAuthentication",
        )

    class TcpCheck(proto.Message):
        r"""Information required for a TCP Uptime check request.

        Attributes:
            port (int):
                The TCP port on the server against which to run the check.
                Will be combined with host (specified within the
                ``monitored_resource``) to construct the full URL. Required.
            ping_config (google.cloud.monitoring_v3.types.UptimeCheckConfig.PingConfig):
                Contains information needed to add pings to a
                TCP check.
        """

        port: int = proto.Field(
            proto.INT32,
            number=1,
        )
        ping_config: "UptimeCheckConfig.PingConfig" = proto.Field(
            proto.MESSAGE,
            number=2,
            message="UptimeCheckConfig.PingConfig",
        )

    class ContentMatcher(proto.Message):
        r"""Optional. Used to perform content matching. This allows
        matching based on substrings and regular expressions, together
        with their negations. Only the first 4&nbsp;MB of an HTTP or
        HTTPS check's response (and the first 1&nbsp;MB of a TCP check's
        response) are examined for purposes of content matching.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            content (str):
                String, regex or JSON content to match. Maximum 1024 bytes.
                An empty ``content`` string indicates no content matching is
                to be performed.
            matcher (google.cloud.monitoring_v3.types.UptimeCheckConfig.ContentMatcher.ContentMatcherOption):
                The type of content matcher that will be applied to the
                server output, compared to the ``content`` string when the
                check is run.
            json_path_matcher (google.cloud.monitoring_v3.types.UptimeCheckConfig.ContentMatcher.JsonPathMatcher):
                Matcher information for ``MATCHES_JSON_PATH`` and
                ``NOT_MATCHES_JSON_PATH``

                This field is a member of `oneof`_ ``additional_matcher_info``.
        """

        class ContentMatcherOption(proto.Enum):
            r"""Options to perform content matching.

            Values:
                CONTENT_MATCHER_OPTION_UNSPECIFIED (0):
                    No content matcher type specified (maintained for backward
                    compatibility, but deprecated for future use). Treated as
                    ``CONTAINS_STRING``.
                CONTAINS_STRING (1):
                    Selects substring matching. The match succeeds if the output
                    contains the ``content`` string. This is the default value
                    for checks without a ``matcher`` option, or where the value
                    of ``matcher`` is ``CONTENT_MATCHER_OPTION_UNSPECIFIED``.
                NOT_CONTAINS_STRING (2):
                    Selects negation of substring matching. The match succeeds
                    if the output does *NOT* contain the ``content`` string.
                MATCHES_REGEX (3):
                    Selects regular-expression matching. The match succeeds if
                    the output matches the regular expression specified in the
                    ``content`` string. Regex matching is only supported for
                    HTTP/HTTPS checks.
                NOT_MATCHES_REGEX (4):
                    Selects negation of regular-expression matching. The match
                    succeeds if the output does *NOT* match the regular
                    expression specified in the ``content`` string. Regex
                    matching is only supported for HTTP/HTTPS checks.
                MATCHES_JSON_PATH (5):
                    Selects JSONPath matching. See ``JsonPathMatcher`` for
                    details on when the match succeeds. JSONPath matching is
                    only supported for HTTP/HTTPS checks.
                NOT_MATCHES_JSON_PATH (6):
                    Selects JSONPath matching. See ``JsonPathMatcher`` for
                    details on when the match succeeds. Succeeds when output
                    does *NOT* match as specified. JSONPath is only supported
                    for HTTP/HTTPS checks.
            """
            CONTENT_MATCHER_OPTION_UNSPECIFIED = 0
            CONTAINS_STRING = 1
            NOT_CONTAINS_STRING = 2
            MATCHES_REGEX = 3
            NOT_MATCHES_REGEX = 4
            MATCHES_JSON_PATH = 5
            NOT_MATCHES_JSON_PATH = 6

        class JsonPathMatcher(proto.Message):
            r"""Information needed to perform a JSONPath content match. Used for
            ``ContentMatcherOption::MATCHES_JSON_PATH`` and
            ``ContentMatcherOption::NOT_MATCHES_JSON_PATH``.

            Attributes:
                json_path (str):
                    JSONPath within the response output pointing to the expected
                    ``ContentMatcher::content`` to match against.
                json_matcher (google.cloud.monitoring_v3.types.UptimeCheckConfig.ContentMatcher.JsonPathMatcher.JsonPathMatcherOption):
                    The type of JSONPath match that will be applied to the JSON
                    output (``ContentMatcher.content``)
            """

            class JsonPathMatcherOption(proto.Enum):
                r"""Options to perform JSONPath content matching.

                Values:
                    JSON_PATH_MATCHER_OPTION_UNSPECIFIED (0):
                        No JSONPath matcher type specified (not
                        valid).
                    EXACT_MATCH (1):
                        Selects 'exact string' matching. The match succeeds if the
                        content at the ``json_path`` within the output is exactly
                        the same as the ``content`` string.
                    REGEX_MATCH (2):
                        Selects regular-expression matching. The match succeeds if
                        the content at the ``json_path`` within the output matches
                        the regular expression specified in the ``content`` string.
                """
                JSON_PATH_MATCHER_OPTION_UNSPECIFIED = 0
                EXACT_MATCH = 1
                REGEX_MATCH = 2

            json_path: str = proto.Field(
                proto.STRING,
                number=1,
            )
            json_matcher: "UptimeCheckConfig.ContentMatcher.JsonPathMatcher.JsonPathMatcherOption" = proto.Field(
                proto.ENUM,
                number=2,
                enum="UptimeCheckConfig.ContentMatcher.JsonPathMatcher.JsonPathMatcherOption",
            )

        content: str = proto.Field(
            proto.STRING,
            number=1,
        )
        matcher: "UptimeCheckConfig.ContentMatcher.ContentMatcherOption" = proto.Field(
            proto.ENUM,
            number=2,
            enum="UptimeCheckConfig.ContentMatcher.ContentMatcherOption",
        )
        json_path_matcher: "UptimeCheckConfig.ContentMatcher.JsonPathMatcher" = (
            proto.Field(
                proto.MESSAGE,
                number=3,
                oneof="additional_matcher_info",
                message="UptimeCheckConfig.ContentMatcher.JsonPathMatcher",
            )
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    monitored_resource: monitored_resource_pb2.MonitoredResource = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="resource",
        message=monitored_resource_pb2.MonitoredResource,
    )
    resource_group: ResourceGroup = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="resource",
        message=ResourceGroup,
    )
    synthetic_monitor: "SyntheticMonitorTarget" = proto.Field(
        proto.MESSAGE,
        number=21,
        oneof="resource",
        message="SyntheticMonitorTarget",
    )
    http_check: HttpCheck = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="check_request_type",
        message=HttpCheck,
    )
    tcp_check: TcpCheck = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="check_request_type",
        message=TcpCheck,
    )
    period: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=7,
        message=duration_pb2.Duration,
    )
    timeout: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=8,
        message=duration_pb2.Duration,
    )
    content_matchers: MutableSequence[ContentMatcher] = proto.RepeatedField(
        proto.MESSAGE,
        number=9,
        message=ContentMatcher,
    )
    checker_type: CheckerType = proto.Field(
        proto.ENUM,
        number=17,
        enum=CheckerType,
    )
    selected_regions: MutableSequence["UptimeCheckRegion"] = proto.RepeatedField(
        proto.ENUM,
        number=10,
        enum="UptimeCheckRegion",
    )
    is_internal: bool = proto.Field(
        proto.BOOL,
        number=15,
    )
    internal_checkers: MutableSequence["InternalChecker"] = proto.RepeatedField(
        proto.MESSAGE,
        number=14,
        message="InternalChecker",
    )
    user_labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=20,
    )


class UptimeCheckIp(proto.Message):
    r"""Contains the region, location, and list of IP
    addresses where checkers in the location run from.

    Attributes:
        region (google.cloud.monitoring_v3.types.UptimeCheckRegion):
            A broad region category in which the IP
            address is located.
        location (str):
            A more specific location within the region
            that typically encodes a particular
            city/town/metro (and its containing
            state/province or country) within the broader
            umbrella region category.
        ip_address (str):
            The IP address from which the Uptime check
            originates. This is a fully specified IP address
            (not an IP address range). Most IP addresses, as
            of this publication, are in IPv4 format;
            however, one should not rely on the IP addresses
            being in IPv4 format indefinitely, and should
            support interpreting this field in either IPv4
            or IPv6 format.
    """

    region: "UptimeCheckRegion" = proto.Field(
        proto.ENUM,
        number=1,
        enum="UptimeCheckRegion",
    )
    location: str = proto.Field(
        proto.STRING,
        number=2,
    )
    ip_address: str = proto.Field(
        proto.STRING,
        number=3,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.api import distribution_pb2  # type: ignore
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "ComparisonType",
        "ServiceTier",
        "TypedValue",
        "TimeInterval",
        "Aggregation",
    },
)


class ComparisonType(proto.Enum):
    r"""Specifies an ordering relationship on two arguments, called ``left``
    and ``right``.

    Values:
        COMPARISON_UNSPECIFIED (0):
            No ordering relationship is specified.
        COMPARISON_GT (1):
            True if the left argument is greater than the
            right argument.
        COMPARISON_GE (2):
            True if the left argument is greater than or
            equal to the right argument.
        COMPARISON_LT (3):
            True if the left argument is less than the
            right argument.
        COMPARISON_LE (4):
            True if the left argument is less than or
            equal to the right argument.
        COMPARISON_EQ (5):
            True if the left argument is equal to the
            right argument.
        COMPARISON_NE (6):
            True if the left argument is not equal to the
            right argument.
    """
    COMPARISON_UNSPECIFIED = 0
    COMPARISON_GT = 1
    COMPARISON_GE = 2
    COMPARISON_LT = 3
    COMPARISON_LE = 4
    COMPARISON_EQ = 5
    COMPARISON_NE = 6


class ServiceTier(proto.Enum):
    r"""The tier of service for a Metrics Scope. Please see the `service
    tiers
    documentation <https://cloud.google.com/monitoring/workspaces/tiers>`__
    for more details.

    Values:
        SERVICE_TIER_UNSPECIFIED (0):
            An invalid sentinel value, used to indicate
            that a tier has not been provided explicitly.
        SERVICE_TIER_BASIC (1):
            The Cloud Monitoring Basic tier, a free tier of service that
            provides basic features, a moderate allotment of logs, and
            access to built-in metrics. A number of features are not
            available in this tier. For more details, see `the service
            tiers
            documentation <https://cloud.google.com/monitoring/workspaces/tiers>`__.
        SERVICE_TIER_PREMIUM (2):
            The Cloud Monitoring Premium tier, a higher, more expensive
            tier of service that provides access to all Cloud Monitoring
            features, lets you use Cloud Monitoring with AWS accounts,
            and has a larger allotments for logs and metrics. For more
            details, see `the service tiers
            documentation <https://cloud.google.com/monitoring/workspaces/tiers>`__.
    """
    _pb_options = {"deprecated": True}
    SERVICE_TIER_UNSPECIFIED = 0
    SERVICE_TIER_BASIC = 1
    SERVICE_TIER_PREMIUM = 2


class TypedValue(proto.Message):
    r"""A single strongly-typed value.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        bool_value (bool):
            A Boolean value: ``true`` or ``false``.

            This field is a member of `oneof`_ ``value``.
        int64_value (int):
            A 64-bit integer. Its range is approximately
            &plusmn;9.2x10<sup>18</sup>.

            This field is a member of `oneof`_ ``value``.
        double_value (float):
            A 64-bit double-precision floating-point
            number. Its magnitude is approximately
            &plusmn;10<sup>&plusmn;300</sup> and it has 16
            significant digits of precision.

            This field is a member of `oneof`_ ``value``.
        string_value (str):
            A variable-length string value.

            This field is a member of `oneof`_ ``value``.
        distribution_value (google.api.distribution_pb2.Distribution):
            A distribution value.

            This field is a member of `oneof`_ ``value``.
    """

    bool_value: bool = proto.Field(
        proto.BOOL,
        number=1,
        oneof="value",
    )
    int64_value: int = proto.Field(
        proto.INT64,
        number=2,
        oneof="value",
    )
    double_value: float = proto.Field(
        proto.DOUBLE,
        number=3,
        oneof="value",
    )
    string_value: str = proto.Field(
        proto.STRING,
        number=4,
        oneof="value",
    )
    distribution_value: distribution_pb2.Distribution = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="value",
        message=distribution_pb2.Distribution,
    )


class TimeInterval(proto.Message):
    r"""Describes a time interval:

    - Reads: A half-open time interval. It includes the end time but
      excludes the start time: ``(startTime, endTime]``. The start time
      must be specified, must be earlier than the end time, and should
      be no older than the data retention period for the metric.
    - Writes: A closed time interval. It extends from the start time to
      the end time, and includes both: ``[startTime, endTime]``. Valid
      time intervals depend on the
      ```MetricKind`` <https://cloud.google.com/monitoring/api/ref_v3/rest/v3/projects.metricDescriptors#MetricKind>`__
      of the metric value. The end time must not be earlier than the
      start time, and the end time must not be more than 25 hours in the
      past or more than five minutes in the future.

      - For ``GAUGE`` metrics, the ``startTime`` value is technically
        optional; if no value is specified, the start time defaults to
        the value of the end time, and the interval represents a single
        point in time. If both start and end times are specified, they
        must be identical. Such an interval is valid only for ``GAUGE``
        metrics, which are point-in-time measurements. The end time of a
        new interval must be at least a millisecond after the end time
        of the previous interval.
      - For ``DELTA`` metrics, the start time and end time must specify
        a non-zero interval, with subsequent points specifying
        contiguous and non-overlapping intervals. For ``DELTA`` metrics,
        the start time of the next interval must be at least a
        millisecond after the end time of the previous interval.
      - For ``CUMULATIVE`` metrics, the start time and end time must
        specify a non-zero interval, with subsequent points specifying
        the same start time and increasing end times, until an event
        resets the cumulative value to zero and sets a new start time
        for the following points. The new start time must be at least a
        millisecond after the end time of the previous interval.
      - The start time of a new interval must be at least a millisecond
        after the end time of the previous interval because intervals
        are closed. If the start time of a new interval is the same as
        the end time of the previous interval, then data written at the
        new start time could overwrite data written at the previous end
        time.

    Attributes:
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Required. The end of the time interval.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. The beginning of the time interval.
            The default value for the start time is the end
            time. The start time must not be later than the
            end time.
    """

    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )


class Aggregation(proto.Message):
    r"""Describes how to combine multiple time series to provide a different
    view of the data. Aggregation of time series is done in two steps.
    First, each time series in the set is *aligned* to the same time
    interval boundaries, then the set of time series is optionally
    *reduced* in number.

    Alignment consists of applying the ``per_series_aligner`` operation
    to each time series after its data has been divided into regular
    ``alignment_period`` time intervals. This process takes *all* of the
    data points in an alignment period, applies a mathematical
    transformation such as averaging, minimum, maximum, delta, etc., and
    converts them into a single data point per period.

    Reduction is when the aligned and transformed time series can
    optionally be combined, reducing the number of time series through
    similar mathematical transformations. Reduction involves applying a
    ``cross_series_reducer`` to all the time series, optionally sorting
    the time series into subsets with ``group_by_fields``, and applying
    the reducer to each subset.

    The raw time series data can contain a huge amount of information
    from multiple sources. Alignment and reduction transforms this mass
    of data into a more manageable and representative collection of
    data, for example "the 95% latency across the average of all tasks
    in a cluster". This representative data can be more easily graphed
    and comprehended, and the individual time series data is still
    available for later drilldown. For more details, see `Filtering and
    aggregation <https://cloud.google.com/monitoring/api/v3/aggregation>`__.

    Attributes:
        alignment_period (google.protobuf.duration_pb2.Duration):
            The ``alignment_period`` specifies a time interval, in
            seconds, that is used to divide the data in all the [time
            series][google.monitoring.v3.TimeSeries] into consistent
            blocks of time. This will be done before the per-series
            aligner can be applied to the data.

            The value must be at least 60 seconds. If a per-series
            aligner other than ``ALIGN_NONE`` is specified, this field
            is required or an error is returned. If no per-series
            aligner is specified, or the aligner ``ALIGN_NONE`` is
            specified, then this field is ignored.

            The maximum value of the ``alignment_period`` is 104 weeks
            (2 years) for charts, and 90,000 seconds (25 hours) for
            alerting policies.
        per_series_aligner (google.cloud.monitoring_v3.types.Aggregation.Aligner):
            An ``Aligner`` describes how to bring the data points in a
            single time series into temporal alignment. Except for
            ``ALIGN_NONE``, all alignments cause all the data points in
            an ``alignment_period`` to be mathematically grouped
            together, resulting in a single data point for each
            ``alignment_period`` with end timestamp at the end of the
            period.

            Not all alignment operations may be applied to all time
            series. The valid choices depend on the ``metric_kind`` and
            ``value_type`` of the original time series. Alignment can
            change the ``metric_kind`` or the ``value_type`` of the time
            series.

            Time series data must be aligned in order to perform
            cross-time series reduction. If ``cross_series_reducer`` is
            specified, then ``per_series_aligner`` must be specified and
            not equal to ``ALIGN_NONE`` and ``alignment_period`` must be
            specified; otherwise, an error is returned.
        cross_series_reducer (google.cloud.monitoring_v3.types.Aggregation.Reducer):
            The reduction operation to be used to combine time series
            into a single time series, where the value of each data
            point in the resulting series is a function of all the
            already aligned values in the input time series.

            Not all reducer operations can be applied to all time
            series. The valid choices depend on the ``metric_kind`` and
            the ``value_type`` of the original time series. Reduction
            can yield a time series with a different ``metric_kind`` or
            ``value_type`` than the input time series.

            Time series data must first be aligned (see
            ``per_series_aligner``) in order to perform cross-time
            series reduction. If ``cross_series_reducer`` is specified,
            then ``per_series_aligner`` must be specified, and must not
            be ``ALIGN_NONE``. An ``alignment_period`` must also be
            specified; otherwise, an error is returned.
        group_by_fields (MutableSequence[str]):
            The set of fields to preserve when ``cross_series_reducer``
            is specified. The ``group_by_fields`` determine how the time
            series are partitioned into subsets prior to applying the
            aggregation operation. Each subset contains time series that
            have the same value for each of the grouping fields. Each
            individual time series is a member of exactly one subset.
            The ``cross_series_reducer`` is applied to each subset of
            time series. It is not possible to reduce across different
            resource types, so this field implicitly contains
            ``resource.type``. Fields not specified in
            ``group_by_fields`` are aggregated away. If
            ``group_by_fields`` is not specified and all the time series
            have the same resource type, then the time series are
            aggregated into a single output time series. If
            ``cross_series_reducer`` is not defined, this field is
            ignored.
    """

    class Aligner(proto.Enum):
        r"""The ``Aligner`` specifies the operation that will be applied to the
        data points in each alignment period in a time series. Except for
        ``ALIGN_NONE``, which specifies that no operation be applied, each
        alignment operation replaces the set of data values in each
        alignment period with a single value: the result of applying the
        operation to the data values. An aligned time series has a single
        data value at the end of each ``alignment_period``.

        An alignment operation can change the data type of the values, too.
        For example, if you apply a counting operation to boolean values,
        the data ``value_type`` in the original time series is ``BOOLEAN``,
        but the ``value_type`` in the aligned result is ``INT64``.

        Values:
            ALIGN_NONE (0):
                No alignment. Raw data is returned. Not valid if
                cross-series reduction is requested. The ``value_type`` of
                the result is the same as the ``value_type`` of the input.
            ALIGN_DELTA (1):
                Align and convert to
                [DELTA][google.api.MetricDescriptor.MetricKind.DELTA]. The
                output is ``delta = y1 - y0``.

                This alignment is valid for
                [CUMULATIVE][google.api.MetricDescriptor.MetricKind.CUMULATIVE]
                and ``DELTA`` metrics. If the selected alignment period
                results in periods with no data, then the aligned value for
                such a period is created by interpolation. The
                ``value_type`` of the aligned result is the same as the
                ``value_type`` of the input.
            ALIGN_RATE (2):
                Align and convert to a rate. The result is computed as
                ``rate = (y1 - y0)/(t1 - t0)``, or "delta over time". Think
                of this aligner as providing the slope of the line that
                passes through the value at the start and at the end of the
                ``alignment_period``.

                This aligner is valid for ``CUMULATIVE`` and ``DELTA``
                metrics with numeric values. If the selected alignment
                period results in periods with no data, then the aligned
                value for such a period is created by interpolation. The
                output is a ``GAUGE`` metric with ``value_type`` ``DOUBLE``.

                If, by "rate", you mean "percentage change", see the
                ``ALIGN_PERCENT_CHANGE`` aligner instead.
            ALIGN_INTERPOLATE (3):
                Align by interpolating between adjacent points around the
                alignment period boundary. This aligner is valid for
                ``GAUGE`` metrics with numeric values. The ``value_type`` of
                the aligned result is the same as the ``value_type`` of the
                input.
            ALIGN_NEXT_OLDER (4):
                Align by moving the most recent data point before the end of
                the alignment period to the boundary at the end of the
                alignment period. This aligner is valid for ``GAUGE``
                metrics. The ``value_type`` of the aligned result is the
                same as the ``value_type`` of the input.
            ALIGN_MIN (10):
                Align the time series by returning the minimum value in each
                alignment period. This aligner is valid for ``GAUGE`` and
                ``DELTA`` metrics with numeric values. The ``value_type`` of
                the aligned result is the same as the ``value_type`` of the
                input.
            ALIGN_MAX (11):
                Align the time series by returning the maximum value in each
                alignment period. This aligner is valid for ``GAUGE`` and
                ``DELTA`` metrics with numeric values. The ``value_type`` of
                the aligned result is the same as the ``value_type`` of the
                input.
            ALIGN_MEAN (12):
                Align the time series by returning the mean value in each
                alignment period. This aligner is valid for ``GAUGE`` and
                ``DELTA`` metrics with numeric values. The ``value_type`` of
                the aligned result is ``DOUBLE``.
            ALIGN_COUNT (13):
                Align the time series by returning the number of values in
                each alignment period. This aligner is valid for ``GAUGE``
                and ``DELTA`` metrics with numeric or Boolean values. The
                ``value_type`` of the aligned result is ``INT64``.
            ALIGN_SUM (14):
                Align the time series by returning the sum of the values in
                each alignment period. This aligner is valid for ``GAUGE``
                and ``DELTA`` metrics with numeric and distribution values.
                The ``value_type`` of the aligned result is the same as the
                ``value_type`` of the input.
            ALIGN_STDDEV (15):
                Align the time series by returning the standard deviation of
                the values in each alignment period. This aligner is valid
                for ``GAUGE`` and ``DELTA`` metrics with numeric values. The
                ``value_type`` of the output is ``DOUBLE``.
            ALIGN_COUNT_TRUE (16):
                Align the time series by returning the number of ``True``
                values in each alignment period. This aligner is valid for
                ``GAUGE`` metrics with Boolean values. The ``value_type`` of
                the output is ``INT64``.
            ALIGN_COUNT_FALSE (24):
                Align the time series by returning the number of ``False``
                values in each alignment period. This aligner is valid for
                ``GAUGE`` metrics with Boolean values. The ``value_type`` of
                the output is ``INT64``.
            ALIGN_FRACTION_TRUE (17):
                Align the time series by returning the ratio of the number
                of ``True`` values to the total number of values in each
                alignment period. This aligner is valid for ``GAUGE``
                metrics with Boolean values. The output value is in the
                range [0.0, 1.0] and has ``value_type`` ``DOUBLE``.
            ALIGN_PERCENTILE_99 (18):
                Align the time series by using `percentile
                aggregation <https://en.wikipedia.org/wiki/Percentile>`__.
                The resulting data point in each alignment period is the
                99th percentile of all data points in the period. This
                aligner is valid for ``GAUGE`` and ``DELTA`` metrics with
                distribution values. The output is a ``GAUGE`` metric with
                ``value_type`` ``DOUBLE``.
            ALIGN_PERCENTILE_95 (19):
                Align the time series by using `percentile
                aggregation <https://en.wikipedia.org/wiki/Percentile>`__.
                The resulting data point in each alignment period is the
                95th percentile of all data points in the period. This
                aligner is valid for ``GAUGE`` and ``DELTA`` metrics with
                distribution values. The output is a ``GAUGE`` metric with
                ``value_type`` ``DOUBLE``.
            ALIGN_PERCENTILE_50 (20):
                Align the time series by using `percentile
                aggregation <https://en.wikipedia.org/wiki/Percentile>`__.
                The resulting data point in each alignment period is the
                50th percentile of all data points in the period. This
                aligner is valid for ``GAUGE`` and ``DELTA`` metrics with
                distribution values. The output is a ``GAUGE`` metric with
                ``value_type`` ``DOUBLE``.
            ALIGN_PERCENTILE_05 (21):
                Align the time series by using `percentile
                aggregation <https://en.wikipedia.org/wiki/Percentile>`__.
                The resulting data point in each alignment period is the 5th
                percentile of all data points in the period. This aligner is
                valid for ``GAUGE`` and ``DELTA`` metrics with distribution
                values. The output is a ``GAUGE`` metric with ``value_type``
                ``DOUBLE``.
            ALIGN_PERCENT_CHANGE (23):
                Align and convert to a percentage change. This aligner is
                valid for ``GAUGE`` and ``DELTA`` metrics with numeric
                values. This alignment returns
                ``((current - previous)/previous) * 100``, where the value
                of ``previous`` is determined based on the
                ``alignment_period``.

                If the values of ``current`` and ``previous`` are both 0,
                then the returned value is 0. If only ``previous`` is 0, the
                returned value is infinity.

                A 10-minute moving mean is computed at each point of the
                alignment period prior to the above calculation to smooth
                the metric and prevent false positives from very short-lived
                spikes. The moving mean is only applicable for data whose
                values are ``>= 0``. Any values ``< 0`` are treated as a
                missing datapoint, and are ignored. While ``DELTA`` metrics
                are accepted by this alignment, special care should be taken
                that the values for the metric will always be positive. The
                output is a ``GAUGE`` metric with ``value_type`` ``DOUBLE``.
        """
        ALIGN_NONE = 0
        ALIGN_DELTA = 1
        ALIGN_RATE = 2
        ALIGN_INTERPOLATE = 3
        ALIGN_NEXT_OLDER = 4
        ALIGN_MIN = 10
        ALIGN_MAX = 11
        ALIGN_MEAN = 12
        ALIGN_COUNT = 13
        ALIGN_SUM = 14
        ALIGN_STDDEV = 15
        ALIGN_COUNT_TRUE = 16
        ALIGN_COUNT_FALSE = 24
        ALIGN_FRACTION_TRUE = 17
        ALIGN_PERCENTILE_99 = 18
        ALIGN_PERCENTILE_95 = 19
        ALIGN_PERCENTILE_50 = 20
        ALIGN_PERCENTILE_05 = 21
        ALIGN_PERCENT_CHANGE = 23

    class Reducer(proto.Enum):
        r"""A Reducer operation describes how to aggregate data points
        from multiple time series into a single time series, where the
        value of each data point in the resulting series is a function
        of all the already aligned values in the input time series.

        Values:
            REDUCE_NONE (0):
                No cross-time series reduction. The output of the
                ``Aligner`` is returned.
            REDUCE_MEAN (1):
                Reduce by computing the mean value across time series for
                each alignment period. This reducer is valid for
                [DELTA][google.api.MetricDescriptor.MetricKind.DELTA] and
                [GAUGE][google.api.MetricDescriptor.MetricKind.GAUGE]
                metrics with numeric or distribution values. The
                ``value_type`` of the output is
                [DOUBLE][google.api.MetricDescriptor.ValueType.DOUBLE].
            REDUCE_MIN (2):
                Reduce by computing the minimum value across time series for
                each alignment period. This reducer is valid for ``DELTA``
                and ``GAUGE`` metrics with numeric values. The
                ``value_type`` of the output is the same as the
                ``value_type`` of the input.
            REDUCE_MAX (3):
                Reduce by computing the maximum value across time series for
                each alignment period. This reducer is valid for ``DELTA``
                and ``GAUGE`` metrics with numeric values. The
                ``value_type`` of the output is the same as the
                ``value_type`` of the input.
            REDUCE_SUM (4):
                Reduce by computing the sum across time series for each
                alignment period. This reducer is valid for ``DELTA`` and
                ``GAUGE`` metrics with numeric and distribution values. The
                ``value_type`` of the output is the same as the
                ``value_type`` of the input.
            REDUCE_STDDEV (5):
                Reduce by computing the standard deviation across time
                series for each alignment period. This reducer is valid for
                ``DELTA`` and ``GAUGE`` metrics with numeric or distribution
                values. The ``value_type`` of the output is ``DOUBLE``.
            REDUCE_COUNT (6):
                Reduce by computing the number of data points across time
                series for each alignment period. This reducer is valid for
                ``DELTA`` and ``GAUGE`` metrics of numeric, Boolean,
                distribution, and string ``value_type``. The ``value_type``
                of the output is ``INT64``.
            REDUCE_COUNT_TRUE (7):
                Reduce by computing the number of ``True``-valued data
                points across time series for each alignment period. This
                reducer is valid for ``DELTA`` and ``GAUGE`` metrics of
                Boolean ``value_type``. The ``value_type`` of the output is
                ``INT64``.
            REDUCE_COUNT_FALSE (15):
                Reduce by computing the number of ``False``-valued data
                points across time series for each alignment period. This
                reducer is valid for ``DELTA`` and ``GAUGE`` metrics of
                Boolean ``value_type``. The ``value_type`` of the output is
                ``INT64``.
            REDUCE_FRACTION_TRUE (8):
                Reduce by computing the ratio of the number of
                ``True``-valued data points to the total number of data
                points for each alignment period. This reducer is valid for
                ``DELTA`` and ``GAUGE`` metrics of Boolean ``value_type``.
                The output value is in the range [0.0, 1.0] and has
                ``value_type`` ``DOUBLE``.
            REDUCE_PERCENTILE_99 (9):
                Reduce by computing the `99th
                percentile <https://en.wikipedia.org/wiki/Percentile>`__ of
                data points across time series for each alignment period.
                This reducer is valid for ``GAUGE`` and ``DELTA`` metrics of
                numeric and distribution type. The value of the output is
                ``DOUBLE``.
            REDUCE_PERCENTILE_95 (10):
                Reduce by computing the `95th
                percentile <https://en.wikipedia.org/wiki/Percentile>`__ of
                data points across time series for each alignment period.
                This reducer is valid for ``GAUGE`` and ``DELTA`` metrics of
                numeric and distribution type. The value of the output is
                ``DOUBLE``.
            REDUCE_PERCENTILE_50 (11):
                Reduce by computing the `50th
                percentile <https://en.wikipedia.org/wiki/Percentile>`__ of
                data points across time series for each alignment period.
                This reducer is valid for ``GAUGE`` and ``DELTA`` metrics of
                numeric and distribution type. The value of the output is
                ``DOUBLE``.
            REDUCE_PERCENTILE_05 (12):
                Reduce by computing the `5th
                percentile <https://en.wikipedia.org/wiki/Percentile>`__ of
                data points across time series for each alignment period.
                This reducer is valid for ``GAUGE`` and ``DELTA`` metrics of
                numeric and distribution type. The value of the output is
                ``DOUBLE``.
        """
        REDUCE_NONE = 0
        REDUCE_MEAN = 1
        REDUCE_MIN = 2
        REDUCE_MAX = 3
        REDUCE_SUM = 4
        REDUCE_STDDEV = 5
        REDUCE_COUNT = 6
        REDUCE_COUNT_TRUE = 7
        REDUCE_COUNT_FALSE = 15
        REDUCE_FRACTION_TRUE = 8
        REDUCE_PERCENTILE_99 = 9
        REDUCE_PERCENTILE_95 = 10
        REDUCE_PERCENTILE_50 = 11
        REDUCE_PERCENTILE_05 = 12

    alignment_period: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=1,
        message=duration_pb2.Duration,
    )
    per_series_aligner: Aligner = proto.Field(
        proto.ENUM,
        number=2,
        enum=Aligner,
    )
    cross_series_reducer: Reducer = proto.Field(
        proto.ENUM,
        number=4,
        enum=Reducer,
    )
    group_by_fields: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2016 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Time series as :mod:`pandas` dataframes."""

import itertools

try:
    import pandas
except ImportError:  # pragma: NO COVER
    pandas = None  # type: ignore[assignment]

from google.cloud import monitoring_v3

TOP_RESOURCE_LABELS = ("project_id", "aws_account", "location", "region", "zone")


def _extract_header(time_series):
    """Return a copy of time_series with the points removed."""
    return monitoring_v3.TimeSeries(
        metric=time_series.metric,
        resource=time_series.resource,
        metric_kind=time_series.metric_kind,
        value_type=time_series.value_type,
    )


def _extract_labels(time_series):
    """Build the combined resource and metric labels, with resource_type."""
    labels = {"resource_type": time_series.resource.type}
    labels.update(time_series.resource.labels)
    labels.update(time_series.metric.labels)
    return labels


def _extract_value(typed_value):
    """Extract the value from a TypedValue."""
    # There is no equivalent of WhichOneOf in proto-plus
    # This may break if the field names have been altered in the
    # proto-plus representation
    # https://github.com/googleapis/proto-plus-python/issues/137
    value_type = monitoring_v3.TypedValue.pb(typed_value).WhichOneof("value")
    return getattr(typed_value, value_type)


def _build_dataframe(time_series_iterable, label=None, labels=None):  # pragma: NO COVER
    """Build a :mod:`pandas` dataframe out of time series.

    :type time_series_iterable:
        iterable over :class:`~google.cloud.monitoring_v3.TimeSeries`
    :param time_series_iterable:
        An iterable (e.g., a query object) yielding time series.

    :type label: str
    :param label:
        (Optional) The label name to use for the dataframe header. This can be
        the name of a resource label or metric label (e.g.,
        ``"instance_name"``), or the string ``"resource_type"``.

    :type labels: list of strings, or None
    :param labels:
        A list or tuple of label names to use for the dataframe header.
        If more than one label name is provided, the resulting dataframe
        will have a multi-level column header.

        Specifying neither ``label`` or ``labels`` results in a dataframe
        with a multi-level column header including the resource type and
        all available resource and metric labels.

        Specifying both ``label`` and ``labels`` is an error.

    :rtype: :class:`pandas.DataFrame`
    :returns: A dataframe where each column represents one time series.

    :raises: :exc:`RuntimeError` if `pandas` is not installed.
    """
    if pandas is None:
        raise RuntimeError("This method requires `pandas` to be installed.")

    if label is not None:
        if labels:
            raise ValueError("Cannot specify both `label` and `labels`.")
        labels = (label,)

    columns = []
    headers = []
    for time_series in time_series_iterable:
        pandas_series = pandas.Series(
            data=[_extract_value(point.value) for point in time_series.points],
            index=[
                point.interval.end_time.timestamp_pb().ToNanoseconds()
                for point in time_series.points
            ],
        )
        columns.append(pandas_series)
        headers.append(_extract_header(time_series))

    # Implement a smart default of using all available labels.
    if labels is None:
        resource_labels = set(
            itertools.chain.from_iterable(header.resource.labels for header in headers)
        )
        metric_labels = set(
            itertools.chain.from_iterable(header.metric.labels for header in headers)
        )
        labels = (
            ["resource_type"]
            + _sorted_resource_labels(resource_labels)
            + sorted(metric_labels)
        )

    # Assemble the columns into a DataFrame.
    dataframe = pandas.DataFrame.from_records(columns).T

    # Convert the timestamp strings into a DatetimeIndex.
    dataframe.index = pandas.to_datetime(dataframe.index)

    # Build a multi-level stack of column headers. Some labels may
    # be undefined for some time series.
    levels = []
    for key in labels:
        level = [_extract_labels(header).get(key, "") for header in headers]
        levels.append(level)

    # Build a column Index or MultiIndex. Do not include level names
    # in the column header if the user requested a single-level header
    # by specifying "label".
    dataframe.columns = pandas.MultiIndex.from_arrays(
        levels, names=labels if not label else None
    )

    # Sort the rows just in case (since the API doesn't guarantee the
    # ordering), and sort the columns lexicographically.
    return dataframe.sort_index(axis=0).sort_index(axis=1)


def _sorted_resource_labels(labels):
    """Sort label names, putting well-known resource labels first."""
    head = [label for label in TOP_RESOURCE_LABELS if label in labels]
    tail = sorted(label for label in labels if label not in TOP_RESOURCE_LABELS)
    return head + tail



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "DroppedLabels",
    },
)


class DroppedLabels(proto.Message):
    r"""A set of (label, value) pairs that were removed from a
    Distribution time series during aggregation and then added as an
    attachment to a Distribution.Exemplar.

    The full label set for the exemplars is constructed by using the
    dropped pairs in combination with the label values that remain
    on the aggregated Distribution time series. The constructed full
    label set can be used to identify the specific entity, such as
    the instance or job, which might be contributing to a long-tail.
    However, with dropped labels, the storage requirements are
    reduced because only the aggregated distribution values for a
    large group of time series are stored.

    Note that there are no guarantees on ordering of the labels from
    exemplar-to-exemplar and from distribution-to-distribution in
    the same stream, and there may be duplicates.  It is up to
    clients to resolve any ambiguities.

    Attributes:
        label (MutableMapping[str, str]):
            Map from label to its value, for all labels
            dropped in any aggregation.
    """

    label: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.api import label_pb2  # type: ignore
from google.api import metric_pb2  # type: ignore
from google.api import monitored_resource_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.monitoring_v3.types import common

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "Point",
        "TimeSeries",
        "TimeSeriesDescriptor",
        "TimeSeriesData",
        "LabelValue",
        "QueryError",
        "TextLocator",
    },
)


class Point(proto.Message):
    r"""A single data point in a time series.

    Attributes:
        interval (google.cloud.monitoring_v3.types.TimeInterval):
            The time interval to which the data point applies. For
            ``GAUGE`` metrics, the start time is optional, but if it is
            supplied, it must equal the end time. For ``DELTA`` metrics,
            the start and end time should specify a non-zero interval,
            with subsequent points specifying contiguous and
            non-overlapping intervals. For ``CUMULATIVE`` metrics, the
            start and end time should specify a non-zero interval, with
            subsequent points specifying the same start time and
            increasing end times, until an event resets the cumulative
            value to zero and sets a new start time for the following
            points.
        value (google.cloud.monitoring_v3.types.TypedValue):
            The value of the data point.
    """

    interval: common.TimeInterval = proto.Field(
        proto.MESSAGE,
        number=1,
        message=common.TimeInterval,
    )
    value: common.TypedValue = proto.Field(
        proto.MESSAGE,
        number=2,
        message=common.TypedValue,
    )


class TimeSeries(proto.Message):
    r"""A collection of data points that describes the time-varying
    values of a metric. A time series is identified by a combination
    of a fully-specified monitored resource and a fully-specified
    metric. This type is used for both listing and creating time
    series.

    Attributes:
        metric (google.api.metric_pb2.Metric):
            The associated metric. A fully-specified
            metric used to identify the time series.
        resource (google.api.monitored_resource_pb2.MonitoredResource):
            The associated monitored resource. Custom metrics can use
            only certain monitored resource types in their time series
            data. For more information, see `Monitored resources for
            custom
            metrics <https://cloud.google.com/monitoring/custom-metrics/creating-metrics#custom-metric-resources>`__.
        metadata (google.api.monitored_resource_pb2.MonitoredResourceMetadata):
            Output only. The associated monitored
            resource metadata. When reading a time series,
            this field will include metadata labels that are
            explicitly named in the reduction. When creating
            a time series, this field is ignored.
        metric_kind (google.api.metric_pb2.MetricKind):
            The metric kind of the time series. When listing time
            series, this metric kind might be different from the metric
            kind of the associated metric if this time series is an
            alignment or reduction of other time series.

            When creating a time series, this field is optional. If
            present, it must be the same as the metric kind of the
            associated metric. If the associated metric's descriptor
            must be auto-created, then this field specifies the metric
            kind of the new descriptor and must be either ``GAUGE`` (the
            default) or ``CUMULATIVE``.
        value_type (google.api.metric_pb2.ValueType):
            The value type of the time series. When listing time series,
            this value type might be different from the value type of
            the associated metric if this time series is an alignment or
            reduction of other time series.

            When creating a time series, this field is optional. If
            present, it must be the same as the type of the data in the
            ``points`` field.
        points (MutableSequence[google.cloud.monitoring_v3.types.Point]):
            The data points of this time series. When listing time
            series, points are returned in reverse time order.

            When creating a time series, this field must contain exactly
            one point and the point's type must be the same as the value
            type of the associated metric. If the associated metric's
            descriptor must be auto-created, then the value type of the
            descriptor is determined by the point's type, which must be
            ``BOOL``, ``INT64``, ``DOUBLE``, or ``DISTRIBUTION``.
        unit (str):
            The units in which the metric value is reported. It is only
            applicable if the ``value_type`` is ``INT64``, ``DOUBLE``,
            or ``DISTRIBUTION``. The ``unit`` defines the representation
            of the stored metric values. This field can only be changed
            through CreateTimeSeries when it is empty.
        description (str):
            Input only. A detailed description of the time series that
            will be associated with the
            [google.api.MetricDescriptor][google.api.MetricDescriptor]
            for the metric. Once set, this field cannot be changed
            through CreateTimeSeries.
    """

    metric: metric_pb2.Metric = proto.Field(
        proto.MESSAGE,
        number=1,
        message=metric_pb2.Metric,
    )
    resource: monitored_resource_pb2.MonitoredResource = proto.Field(
        proto.MESSAGE,
        number=2,
        message=monitored_resource_pb2.MonitoredResource,
    )
    metadata: monitored_resource_pb2.MonitoredResourceMetadata = proto.Field(
        proto.MESSAGE,
        number=7,
        message=monitored_resource_pb2.MonitoredResourceMetadata,
    )
    metric_kind: metric_pb2.MetricDescriptor.MetricKind = proto.Field(
        proto.ENUM,
        number=3,
        enum=metric_pb2.MetricDescriptor.MetricKind,
    )
    value_type: metric_pb2.MetricDescriptor.ValueType = proto.Field(
        proto.ENUM,
        number=4,
        enum=metric_pb2.MetricDescriptor.ValueType,
    )
    points: MutableSequence["Point"] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message="Point",
    )
    unit: str = proto.Field(
        proto.STRING,
        number=8,
    )
    description: str = proto.Field(
        proto.STRING,
        number=9,
    )


class TimeSeriesDescriptor(proto.Message):
    r"""A descriptor for the labels and points in a time series.

    Attributes:
        label_descriptors (MutableSequence[google.api.label_pb2.LabelDescriptor]):
            Descriptors for the labels.
        point_descriptors (MutableSequence[google.cloud.monitoring_v3.types.TimeSeriesDescriptor.ValueDescriptor]):
            Descriptors for the point data value columns.
    """

    class ValueDescriptor(proto.Message):
        r"""A descriptor for the value columns in a data point.

        Attributes:
            key (str):
                The value key.
            value_type (google.api.metric_pb2.ValueType):
                The value type.
            metric_kind (google.api.metric_pb2.MetricKind):
                The value stream kind.
            unit (str):
                The unit in which ``time_series`` point values are reported.
                ``unit`` follows the UCUM format for units as seen in
                https://unitsofmeasure.org/ucum.html. ``unit`` is only valid
                if ``value_type`` is INTEGER, DOUBLE, DISTRIBUTION.
        """

        key: str = proto.Field(
            proto.STRING,
            number=1,
        )
        value_type: metric_pb2.MetricDescriptor.ValueType = proto.Field(
            proto.ENUM,
            number=2,
            enum=metric_pb2.MetricDescriptor.ValueType,
        )
        metric_kind: metric_pb2.MetricDescriptor.MetricKind = proto.Field(
            proto.ENUM,
            number=3,
            enum=metric_pb2.MetricDescriptor.MetricKind,
        )
        unit: str = proto.Field(
            proto.STRING,
            number=4,
        )

    label_descriptors: MutableSequence[label_pb2.LabelDescriptor] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=label_pb2.LabelDescriptor,
    )
    point_descriptors: MutableSequence[ValueDescriptor] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=ValueDescriptor,
    )


class TimeSeriesData(proto.Message):
    r"""Represents the values of a time series associated with a
    TimeSeriesDescriptor.

    Attributes:
        label_values (MutableSequence[google.cloud.monitoring_v3.types.LabelValue]):
            The values of the labels in the time series identifier,
            given in the same order as the ``label_descriptors`` field
            of the TimeSeriesDescriptor associated with this object.
            Each value must have a value of the type given in the
            corresponding entry of ``label_descriptors``.
        point_data (MutableSequence[google.cloud.monitoring_v3.types.TimeSeriesData.PointData]):
            The points in the time series.
    """

    class PointData(proto.Message):
        r"""A point's value columns and time interval. Each point has one or
        more point values corresponding to the entries in
        ``point_descriptors`` field in the TimeSeriesDescriptor associated
        with this object.

        Attributes:
            values (MutableSequence[google.cloud.monitoring_v3.types.TypedValue]):
                The values that make up the point.
            time_interval (google.cloud.monitoring_v3.types.TimeInterval):
                The time interval associated with the point.
        """

        values: MutableSequence[common.TypedValue] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=common.TypedValue,
        )
        time_interval: common.TimeInterval = proto.Field(
            proto.MESSAGE,
            number=2,
            message=common.TimeInterval,
        )

    label_values: MutableSequence["LabelValue"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="LabelValue",
    )
    point_data: MutableSequence[PointData] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=PointData,
    )


class LabelValue(proto.Message):
    r"""A label value.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        bool_value (bool):
            A bool label value.

            This field is a member of `oneof`_ ``value``.
        int64_value (int):
            An int64 label value.

            This field is a member of `oneof`_ ``value``.
        string_value (str):
            A string label value.

            This field is a member of `oneof`_ ``value``.
    """

    bool_value: bool = proto.Field(
        proto.BOOL,
        number=1,
        oneof="value",
    )
    int64_value: int = proto.Field(
        proto.INT64,
        number=2,
        oneof="value",
    )
    string_value: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="value",
    )


class QueryError(proto.Message):
    r"""An error associated with a query in the time series query
    language format.

    Attributes:
        locator (google.cloud.monitoring_v3.types.TextLocator):
            The location of the time series query
            language text that this error applies to.
        message (str):
            The error message.
    """

    locator: "TextLocator" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="TextLocator",
    )
    message: str = proto.Field(
        proto.STRING,
        number=2,
    )


class TextLocator(proto.Message):
    r"""A locator for text. Indicates a particular part of the text of a
    request or of an object referenced in the request.

    For example, suppose the request field ``text`` contains:

    text: "The quick brown fox jumps over the lazy dog."

    Then the locator:

    source: "text" start_position { line: 1 column: 17 } end_position {
    line: 1 column: 19 }

    refers to the part of the text: "fox".

    Attributes:
        source (str):
            The source of the text. The source may be a field in the
            request, in which case its format is the format of the
            google.rpc.BadRequest.FieldViolation.field field in
            https://cloud.google.com/apis/design/errors#error_details.
            It may also be be a source other than the request field
            (e.g. a macro definition referenced in the text of the
            query), in which case this is the name of the source (e.g.
            the macro name).
        start_position (google.cloud.monitoring_v3.types.TextLocator.Position):
            The position of the first byte within the
            text.
        end_position (google.cloud.monitoring_v3.types.TextLocator.Position):
            The position of the last byte within the
            text.
        nested_locator (google.cloud.monitoring_v3.types.TextLocator):
            If ``source``, ``start_position``, and ``end_position``
            describe a call on some object (e.g. a macro in the time
            series query language text) and a location is to be
            designated in that object's text, ``nested_locator``
            identifies the location within that object.
        nesting_reason (str):
            When ``nested_locator`` is set, this field gives the reason
            for the nesting. Usually, the reason is a macro invocation.
            In that case, the macro name (including the leading '@')
            signals the location of the macro call in the text and a
            macro argument name (including the leading '$') signals the
            location of the macro argument inside the macro body that
            got substituted away.
    """

    class Position(proto.Message):
        r"""The position of a byte within the text.

        Attributes:
            line (int):
                The line, starting with 1, where the byte is
                positioned.
            column (int):
                The column within the line, starting with 1,
                where the byte is positioned. This is a byte
                index even though the text is UTF-8.
        """

        line: int = proto.Field(
            proto.INT32,
            number=1,
        )
        column: int = proto.Field(
            proto.INT32,
            number=2,
        )

    source: str = proto.Field(
        proto.STRING,
        number=1,
    )
    start_position: Position = proto.Field(
        proto.MESSAGE,
        number=2,
        message=Position,
    )
    end_position: Position = proto.Field(
        proto.MESSAGE,
        number=3,
        message=Position,
    )
    nested_locator: "TextLocator" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="TextLocator",
    )
    nesting_reason: str = proto.Field(
        proto.STRING,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import duration_pb2  # type: ignore
from google.type import calendar_period_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "Service",
        "ServiceLevelObjective",
        "ServiceLevelIndicator",
        "BasicSli",
        "Range",
        "RequestBasedSli",
        "TimeSeriesRatio",
        "DistributionCut",
        "WindowsBasedSli",
    },
)


class Service(proto.Message):
    r"""A ``Service`` is a discrete, autonomous, and network-accessible
    unit, designed to solve an individual concern
    (`Wikipedia <https://en.wikipedia.org/wiki/Service-orientation>`__).
    In Cloud Monitoring, a ``Service`` acts as the root resource under
    which operational aspects of the service are accessible.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Identifier. Resource name for this Service. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]
        display_name (str):
            Name used for UI elements listing this
            Service.
        custom (google.cloud.monitoring_v3.types.Service.Custom):
            Custom service type.

            This field is a member of `oneof`_ ``identifier``.
        app_engine (google.cloud.monitoring_v3.types.Service.AppEngine):
            Type used for App Engine services.

            This field is a member of `oneof`_ ``identifier``.
        cloud_endpoints (google.cloud.monitoring_v3.types.Service.CloudEndpoints):
            Type used for Cloud Endpoints services.

            This field is a member of `oneof`_ ``identifier``.
        cluster_istio (google.cloud.monitoring_v3.types.Service.ClusterIstio):
            Type used for Istio services that live in a
            Kubernetes cluster.

            This field is a member of `oneof`_ ``identifier``.
        mesh_istio (google.cloud.monitoring_v3.types.Service.MeshIstio):
            Type used for Istio services scoped to an
            Istio mesh.

            This field is a member of `oneof`_ ``identifier``.
        istio_canonical_service (google.cloud.monitoring_v3.types.Service.IstioCanonicalService):
            Type used for canonical services scoped to an Istio mesh.
            Metrics for Istio are `documented
            here <https://istio.io/latest/docs/reference/config/metrics/>`__

            This field is a member of `oneof`_ ``identifier``.
        cloud_run (google.cloud.monitoring_v3.types.Service.CloudRun):
            Type used for Cloud Run services.

            This field is a member of `oneof`_ ``identifier``.
        gke_namespace (google.cloud.monitoring_v3.types.Service.GkeNamespace):
            Type used for GKE Namespaces.

            This field is a member of `oneof`_ ``identifier``.
        gke_workload (google.cloud.monitoring_v3.types.Service.GkeWorkload):
            Type used for GKE Workloads.

            This field is a member of `oneof`_ ``identifier``.
        gke_service (google.cloud.monitoring_v3.types.Service.GkeService):
            Type used for GKE Services (the Kubernetes
            concept of a service).

            This field is a member of `oneof`_ ``identifier``.
        basic_service (google.cloud.monitoring_v3.types.Service.BasicService):
            Message that contains the service type and service labels of
            this service if it is a basic service. Documentation and
            examples
            `here <https://cloud.google.com/stackdriver/docs/solutions/slo-monitoring/api/api-structures#basic-svc-w-basic-sli>`__.
        telemetry (google.cloud.monitoring_v3.types.Service.Telemetry):
            Configuration for how to query telemetry on a
            Service.
        user_labels (MutableMapping[str, str]):
            Labels which have been used to annotate the
            service. Label keys must start with a letter.
            Label keys and values may contain lowercase
            letters, numbers, underscores, and dashes. Label
            keys and values have a maximum length of 63
            characters, and must be less than 128 bytes in
            size. Up to 64 label entries may be stored. For
            labels which do not have a semantic value, the
            empty string may be supplied for the label
            value.
    """

    class Custom(proto.Message):
        r"""Use a custom service to designate a service that you want to
        monitor when none of the other service types (like App Engine,
        Cloud Run, or a GKE type) matches your intended service.

        """

    class AppEngine(proto.Message):
        r"""App Engine service. Learn more at
        https://cloud.google.com/appengine.

        Attributes:
            module_id (str):
                The ID of the App Engine module underlying this service.
                Corresponds to the ``module_id`` resource label in the
                ```gae_app`` monitored
                resource <https://cloud.google.com/monitoring/api/resources#tag_gae_app>`__.
        """

        module_id: str = proto.Field(
            proto.STRING,
            number=1,
        )

    class CloudEndpoints(proto.Message):
        r"""Cloud Endpoints service. Learn more at
        https://cloud.google.com/endpoints.

        Attributes:
            service (str):
                The name of the Cloud Endpoints service underlying this
                service. Corresponds to the ``service`` resource label in
                the ```api`` monitored
                resource <https://cloud.google.com/monitoring/api/resources#tag_api>`__.
        """

        service: str = proto.Field(
            proto.STRING,
            number=1,
        )

    class ClusterIstio(proto.Message):
        r"""Istio service scoped to a single Kubernetes cluster. Learn
        more at https://istio.io. Clusters running OSS Istio will have
        their services ingested as this type.

        Attributes:
            location (str):
                The location of the Kubernetes cluster in which this Istio
                service is defined. Corresponds to the ``location`` resource
                label in ``k8s_cluster`` resources.
            cluster_name (str):
                The name of the Kubernetes cluster in which this Istio
                service is defined. Corresponds to the ``cluster_name``
                resource label in ``k8s_cluster`` resources.
            service_namespace (str):
                The namespace of the Istio service underlying this service.
                Corresponds to the ``destination_service_namespace`` metric
                label in Istio metrics.
            service_name (str):
                The name of the Istio service underlying this service.
                Corresponds to the ``destination_service_name`` metric label
                in Istio metrics.
        """

        location: str = proto.Field(
            proto.STRING,
            number=1,
        )
        cluster_name: str = proto.Field(
            proto.STRING,
            number=2,
        )
        service_namespace: str = proto.Field(
            proto.STRING,
            number=3,
        )
        service_name: str = proto.Field(
            proto.STRING,
            number=4,
        )

    class MeshIstio(proto.Message):
        r"""Istio service scoped to an Istio mesh. Anthos clusters
        running ASM < 1.6.8 will have their services ingested as this
        type.

        Attributes:
            mesh_uid (str):
                Identifier for the mesh in which this Istio service is
                defined. Corresponds to the ``mesh_uid`` metric label in
                Istio metrics.
            service_namespace (str):
                The namespace of the Istio service underlying this service.
                Corresponds to the ``destination_service_namespace`` metric
                label in Istio metrics.
            service_name (str):
                The name of the Istio service underlying this service.
                Corresponds to the ``destination_service_name`` metric label
                in Istio metrics.
        """

        mesh_uid: str = proto.Field(
            proto.STRING,
            number=1,
        )
        service_namespace: str = proto.Field(
            proto.STRING,
            number=3,
        )
        service_name: str = proto.Field(
            proto.STRING,
            number=4,
        )

    class IstioCanonicalService(proto.Message):
        r"""Canonical service scoped to an Istio mesh. Anthos clusters
        running ASM >= 1.6.8 will have their services ingested as this
        type.

        Attributes:
            mesh_uid (str):
                Identifier for the Istio mesh in which this canonical
                service is defined. Corresponds to the ``mesh_uid`` metric
                label in `Istio
                metrics <https://cloud.google.com/monitoring/api/metrics_istio>`__.
            canonical_service_namespace (str):
                The namespace of the canonical service underlying this
                service. Corresponds to the
                ``destination_canonical_service_namespace`` metric label in
                `Istio
                metrics <https://cloud.google.com/monitoring/api/metrics_istio>`__.
            canonical_service (str):
                The name of the canonical service underlying this service.
                Corresponds to the ``destination_canonical_service_name``
                metric label in label in `Istio
                metrics <https://cloud.google.com/monitoring/api/metrics_istio>`__.
        """

        mesh_uid: str = proto.Field(
            proto.STRING,
            number=1,
        )
        canonical_service_namespace: str = proto.Field(
            proto.STRING,
            number=3,
        )
        canonical_service: str = proto.Field(
            proto.STRING,
            number=4,
        )

    class CloudRun(proto.Message):
        r"""Cloud Run service. Learn more at
        https://cloud.google.com/run.

        Attributes:
            service_name (str):
                The name of the Cloud Run service. Corresponds to the
                ``service_name`` resource label in the
                ```cloud_run_revision`` monitored
                resource <https://cloud.google.com/monitoring/api/resources#tag_cloud_run_revision>`__.
            location (str):
                The location the service is run. Corresponds to the
                ``location`` resource label in the ```cloud_run_revision``
                monitored
                resource <https://cloud.google.com/monitoring/api/resources#tag_cloud_run_revision>`__.
        """

        service_name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        location: str = proto.Field(
            proto.STRING,
            number=2,
        )

    class GkeNamespace(proto.Message):
        r"""GKE Namespace. The field names correspond to the resource metadata
        labels on monitored resources that fall under a namespace (for
        example, ``k8s_container`` or ``k8s_pod``).

        Attributes:
            project_id (str):
                Output only. The project this resource lives in. For legacy
                services migrated from the ``Custom`` type, this may be a
                distinct project from the one parenting the service itself.
            location (str):
                The location of the parent cluster. This may
                be a zone or region.
            cluster_name (str):
                The name of the parent cluster.
            namespace_name (str):
                The name of this namespace.
        """

        project_id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        location: str = proto.Field(
            proto.STRING,
            number=2,
        )
        cluster_name: str = proto.Field(
            proto.STRING,
            number=3,
        )
        namespace_name: str = proto.Field(
            proto.STRING,
            number=4,
        )

    class GkeWorkload(proto.Message):
        r"""A GKE Workload (Deployment, StatefulSet, etc). The field names
        correspond to the metadata labels on monitored resources that fall
        under a workload (for example, ``k8s_container`` or ``k8s_pod``).

        Attributes:
            project_id (str):
                Output only. The project this resource lives in. For legacy
                services migrated from the ``Custom`` type, this may be a
                distinct project from the one parenting the service itself.
            location (str):
                The location of the parent cluster. This may
                be a zone or region.
            cluster_name (str):
                The name of the parent cluster.
            namespace_name (str):
                The name of the parent namespace.
            top_level_controller_type (str):
                The type of this workload (for example,
                "Deployment" or "DaemonSet")
            top_level_controller_name (str):
                The name of this workload.
        """

        project_id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        location: str = proto.Field(
            proto.STRING,
            number=2,
        )
        cluster_name: str = proto.Field(
            proto.STRING,
            number=3,
        )
        namespace_name: str = proto.Field(
            proto.STRING,
            number=4,
        )
        top_level_controller_type: str = proto.Field(
            proto.STRING,
            number=5,
        )
        top_level_controller_name: str = proto.Field(
            proto.STRING,
            number=6,
        )

    class GkeService(proto.Message):
        r"""GKE Service. The "service" here represents a `Kubernetes service
        object <https://kubernetes.io/docs/concepts/services-networking/service>`__.
        The field names correspond to the resource labels on
        ```k8s_service`` monitored
        resources <https://cloud.google.com/monitoring/api/resources#tag_k8s_service>`__.

        Attributes:
            project_id (str):
                Output only. The project this resource lives in. For legacy
                services migrated from the ``Custom`` type, this may be a
                distinct project from the one parenting the service itself.
            location (str):
                The location of the parent cluster. This may
                be a zone or region.
            cluster_name (str):
                The name of the parent cluster.
            namespace_name (str):
                The name of the parent namespace.
            service_name (str):
                The name of this service.
        """

        project_id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        location: str = proto.Field(
            proto.STRING,
            number=2,
        )
        cluster_name: str = proto.Field(
            proto.STRING,
            number=3,
        )
        namespace_name: str = proto.Field(
            proto.STRING,
            number=4,
        )
        service_name: str = proto.Field(
            proto.STRING,
            number=5,
        )

    class BasicService(proto.Message):
        r"""A well-known service type, defined by its service type and service
        labels. Documentation and examples
        `here <https://cloud.google.com/stackdriver/docs/solutions/slo-monitoring/api/api-structures#basic-svc-w-basic-sli>`__.

        Attributes:
            service_type (str):
                The type of service that this basic service defines, e.g.
                APP_ENGINE service type. Documentation and valid values
                `here <https://cloud.google.com/stackdriver/docs/solutions/slo-monitoring/api/api-structures#basic-svc-w-basic-sli>`__.
            service_labels (MutableMapping[str, str]):
                Labels that specify the resource that emits the monitoring
                data which is used for SLO reporting of this ``Service``.
                Documentation and valid values for given service types
                `here <https://cloud.google.com/stackdriver/docs/solutions/slo-monitoring/api/api-structures#basic-svc-w-basic-sli>`__.
        """

        service_type: str = proto.Field(
            proto.STRING,
            number=1,
        )
        service_labels: MutableMapping[str, str] = proto.MapField(
            proto.STRING,
            proto.STRING,
            number=2,
        )

    class Telemetry(proto.Message):
        r"""Configuration for how to query telemetry on a Service.

        Attributes:
            resource_name (str):
                The full name of the resource that defines this service.
                Formatted as described in
                https://cloud.google.com/apis/design/resource_names.
        """

        resource_name: str = proto.Field(
            proto.STRING,
            number=1,
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    custom: Custom = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="identifier",
        message=Custom,
    )
    app_engine: AppEngine = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="identifier",
        message=AppEngine,
    )
    cloud_endpoints: CloudEndpoints = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="identifier",
        message=CloudEndpoints,
    )
    cluster_istio: ClusterIstio = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="identifier",
        message=ClusterIstio,
    )
    mesh_istio: MeshIstio = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="identifier",
        message=MeshIstio,
    )
    istio_canonical_service: IstioCanonicalService = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="identifier",
        message=IstioCanonicalService,
    )
    cloud_run: CloudRun = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="identifier",
        message=CloudRun,
    )
    gke_namespace: GkeNamespace = proto.Field(
        proto.MESSAGE,
        number=15,
        oneof="identifier",
        message=GkeNamespace,
    )
    gke_workload: GkeWorkload = proto.Field(
        proto.MESSAGE,
        number=16,
        oneof="identifier",
        message=GkeWorkload,
    )
    gke_service: GkeService = proto.Field(
        proto.MESSAGE,
        number=17,
        oneof="identifier",
        message=GkeService,
    )
    basic_service: BasicService = proto.Field(
        proto.MESSAGE,
        number=19,
        message=BasicService,
    )
    telemetry: Telemetry = proto.Field(
        proto.MESSAGE,
        number=13,
        message=Telemetry,
    )
    user_labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=14,
    )


class ServiceLevelObjective(proto.Message):
    r"""A Service-Level Objective (SLO) describes a level of desired
    good service. It consists of a service-level indicator (SLI), a
    performance goal, and a period over which the objective is to be
    evaluated against that goal. The SLO can use SLIs defined in a
    number of different manners. Typical SLOs might include "99% of
    requests in each rolling week have latency below 200
    milliseconds" or "99.5% of requests in each calendar month
    return successfully."

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Identifier. Resource name for this
            ``ServiceLevelObjective``. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/services/[SERVICE_ID]/serviceLevelObjectives/[SLO_NAME]
        display_name (str):
            Name used for UI elements listing this SLO.
        service_level_indicator (google.cloud.monitoring_v3.types.ServiceLevelIndicator):
            The definition of good service, used to measure and
            calculate the quality of the ``Service``'s performance with
            respect to a single aspect of service quality.
        goal (float):
            The fraction of service that must be good in order for this
            objective to be met. ``0 < goal <= 0.9999``.
        rolling_period (google.protobuf.duration_pb2.Duration):
            A rolling time period, semantically "in the past
            ``<rolling_period>``". Must be an integer multiple of 1 day
            no larger than 30 days.

            This field is a member of `oneof`_ ``period``.
        calendar_period (google.type.calendar_period_pb2.CalendarPeriod):
            A calendar period, semantically "since the start of the
            current ``<calendar_period>``". At this time, only ``DAY``,
            ``WEEK``, ``FORTNIGHT``, and ``MONTH`` are supported.

            This field is a member of `oneof`_ ``period``.
        user_labels (MutableMapping[str, str]):
            Labels which have been used to annotate the
            service-level objective. Label keys must start
            with a letter. Label keys and values may contain
            lowercase letters, numbers, underscores, and
            dashes. Label keys and values have a maximum
            length of 63 characters, and must be less than
            128 bytes in size. Up to 64 label entries may be
            stored. For labels which do not have a semantic
            value, the empty string may be supplied for the
            label value.
    """

    class View(proto.Enum):
        r"""``ServiceLevelObjective.View`` determines what form of
        ``ServiceLevelObjective`` is returned from
        ``GetServiceLevelObjective``, ``ListServiceLevelObjectives``, and
        ``ListServiceLevelObjectiveVersions`` RPCs.

        Values:
            VIEW_UNSPECIFIED (0):
                Same as FULL.
            FULL (2):
                Return the embedded ``ServiceLevelIndicator`` in the form in
                which it was defined. If it was defined using a
                ``BasicSli``, return that ``BasicSli``.
            EXPLICIT (1):
                For ``ServiceLevelIndicator``\ s using ``BasicSli``
                articulation, instead return the ``ServiceLevelIndicator``
                with its mode of computation fully spelled out as a
                ``RequestBasedSli``. For ``ServiceLevelIndicator``\ s using
                ``RequestBasedSli`` or ``WindowsBasedSli``, return the
                ``ServiceLevelIndicator`` as it was provided.
        """
        VIEW_UNSPECIFIED = 0
        FULL = 2
        EXPLICIT = 1

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=11,
    )
    service_level_indicator: "ServiceLevelIndicator" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="ServiceLevelIndicator",
    )
    goal: float = proto.Field(
        proto.DOUBLE,
        number=4,
    )
    rolling_period: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="period",
        message=duration_pb2.Duration,
    )
    calendar_period: calendar_period_pb2.CalendarPeriod = proto.Field(
        proto.ENUM,
        number=6,
        oneof="period",
        enum=calendar_period_pb2.CalendarPeriod,
    )
    user_labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=12,
    )


class ServiceLevelIndicator(proto.Message):
    r"""A Service-Level Indicator (SLI) describes the "performance" of a
    service. For some services, the SLI is well-defined. In such cases,
    the SLI can be described easily by referencing the well-known SLI
    and providing the needed parameters. Alternatively, a "custom" SLI
    can be defined with a query to the underlying metric store. An SLI
    is defined to be ``good_service / total_service`` over any queried
    time interval. The value of performance always falls into the range
    ``0 <= performance <= 1``. A custom SLI describes how to compute
    this ratio, whether this is by dividing values from a pair of time
    series, cutting a ``Distribution`` into good and bad counts, or
    counting time windows in which the service complies with a
    criterion. For separation of concerns, a single Service-Level
    Indicator measures performance for only one aspect of service
    quality, such as fraction of successful queries or fast-enough
    queries.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        basic_sli (google.cloud.monitoring_v3.types.BasicSli):
            Basic SLI on a well-known service type.

            This field is a member of `oneof`_ ``type``.
        request_based (google.cloud.monitoring_v3.types.RequestBasedSli):
            Request-based SLIs

            This field is a member of `oneof`_ ``type``.
        windows_based (google.cloud.monitoring_v3.types.WindowsBasedSli):
            Windows-based SLIs

            This field is a member of `oneof`_ ``type``.
    """

    basic_sli: "BasicSli" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="type",
        message="BasicSli",
    )
    request_based: "RequestBasedSli" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="type",
        message="RequestBasedSli",
    )
    windows_based: "WindowsBasedSli" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="type",
        message="WindowsBasedSli",
    )


class BasicSli(proto.Message):
    r"""An SLI measuring performance on a well-known service type.
    Performance will be computed on the basis of pre-defined metrics.
    The type of the ``service_resource`` determines the metrics to use
    and the ``service_resource.labels`` and ``metric_labels`` are used
    to construct a monitoring filter to filter that metric down to just
    the data relevant to this service.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        method (MutableSequence[str]):
            OPTIONAL: The set of RPCs to which this SLI
            is relevant. Telemetry from other methods will
            not be used to calculate performance for this
            SLI. If omitted, this SLI applies to all the
            Service's methods. For service types that don't
            support breaking down by method, setting this
            field will result in an error.
        location (MutableSequence[str]):
            OPTIONAL: The set of locations to which this
            SLI is relevant. Telemetry from other locations
            will not be used to calculate performance for
            this SLI. If omitted, this SLI applies to all
            locations in which the Service has activity. For
            service types that don't support breaking down
            by location, setting this field will result in
            an error.
        version (MutableSequence[str]):
            OPTIONAL: The set of API versions to which
            this SLI is relevant. Telemetry from other API
            versions will not be used to calculate
            performance for this SLI. If omitted, this SLI
            applies to all API versions. For service types
            that don't support breaking down by version,
            setting this field will result in an error.
        availability (google.cloud.monitoring_v3.types.BasicSli.AvailabilityCriteria):
            Good service is defined to be the count of
            requests made to this service that return
            successfully.

            This field is a member of `oneof`_ ``sli_criteria``.
        latency (google.cloud.monitoring_v3.types.BasicSli.LatencyCriteria):
            Good service is defined to be the count of requests made to
            this service that are fast enough with respect to
            ``latency.threshold``.

            This field is a member of `oneof`_ ``sli_criteria``.
    """

    class AvailabilityCriteria(proto.Message):
        r"""Future parameters for the availability SLI."""

    class LatencyCriteria(proto.Message):
        r"""Parameters for a latency threshold SLI.

        Attributes:
            threshold (google.protobuf.duration_pb2.Duration):
                Good service is defined to be the count of requests made to
                this service that return in no more than ``threshold``.
        """

        threshold: duration_pb2.Duration = proto.Field(
            proto.MESSAGE,
            number=3,
            message=duration_pb2.Duration,
        )

    method: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )
    location: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=8,
    )
    version: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=9,
    )
    availability: AvailabilityCriteria = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="sli_criteria",
        message=AvailabilityCriteria,
    )
    latency: LatencyCriteria = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="sli_criteria",
        message=LatencyCriteria,
    )


class Range(proto.Message):
    r"""Range of numerical values within ``min`` and ``max``.

    Attributes:
        min_ (float):
            Range minimum.
        max_ (float):
            Range maximum.
    """

    min_: float = proto.Field(
        proto.DOUBLE,
        number=1,
    )
    max_: float = proto.Field(
        proto.DOUBLE,
        number=2,
    )


class RequestBasedSli(proto.Message):
    r"""Service Level Indicators for which atomic units of service
    are counted directly.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        good_total_ratio (google.cloud.monitoring_v3.types.TimeSeriesRatio):
            ``good_total_ratio`` is used when the ratio of
            ``good_service`` to ``total_service`` is computed from two
            ``TimeSeries``.

            This field is a member of `oneof`_ ``method``.
        distribution_cut (google.cloud.monitoring_v3.types.DistributionCut):
            ``distribution_cut`` is used when ``good_service`` is a
            count of values aggregated in a ``Distribution`` that fall
            into a good range. The ``total_service`` is the total count
            of all values aggregated in the ``Distribution``.

            This field is a member of `oneof`_ ``method``.
    """

    good_total_ratio: "TimeSeriesRatio" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="method",
        message="TimeSeriesRatio",
    )
    distribution_cut: "DistributionCut" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="method",
        message="DistributionCut",
    )


class TimeSeriesRatio(proto.Message):
    r"""A ``TimeSeriesRatio`` specifies two ``TimeSeries`` to use for
    computing the ``good_service / total_service`` ratio. The specified
    ``TimeSeries`` must have ``ValueType = DOUBLE`` or
    ``ValueType = INT64`` and must have ``MetricKind = DELTA`` or
    ``MetricKind = CUMULATIVE``. The ``TimeSeriesRatio`` must specify
    exactly two of good, bad, and total, and the relationship
    ``good_service + bad_service = total_service`` will be assumed.

    Attributes:
        good_service_filter (str):
            A `monitoring
            filter <https://cloud.google.com/monitoring/api/v3/filters>`__
            specifying a ``TimeSeries`` quantifying good service
            provided. Must have ``ValueType = DOUBLE`` or
            ``ValueType = INT64`` and must have ``MetricKind = DELTA``
            or ``MetricKind = CUMULATIVE``.
        bad_service_filter (str):
            A `monitoring
            filter <https://cloud.google.com/monitoring/api/v3/filters>`__
            specifying a ``TimeSeries`` quantifying bad service, either
            demanded service that was not provided or demanded service
            that was of inadequate quality. Must have
            ``ValueType = DOUBLE`` or ``ValueType = INT64`` and must
            have ``MetricKind = DELTA`` or ``MetricKind = CUMULATIVE``.
        total_service_filter (str):
            A `monitoring
            filter <https://cloud.google.com/monitoring/api/v3/filters>`__
            specifying a ``TimeSeries`` quantifying total demanded
            service. Must have ``ValueType = DOUBLE`` or
            ``ValueType = INT64`` and must have ``MetricKind = DELTA``
            or ``MetricKind = CUMULATIVE``.
    """

    good_service_filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    bad_service_filter: str = proto.Field(
        proto.STRING,
        number=5,
    )
    total_service_filter: str = proto.Field(
        proto.STRING,
        number=6,
    )


class DistributionCut(proto.Message):
    r"""A ``DistributionCut`` defines a ``TimeSeries`` and thresholds used
    for measuring good service and total service. The ``TimeSeries``
    must have ``ValueType = DISTRIBUTION`` and ``MetricKind = DELTA`` or
    ``MetricKind = CUMULATIVE``. The computed ``good_service`` will be
    the estimated count of values in the ``Distribution`` that fall
    within the specified ``min`` and ``max``.

    Attributes:
        distribution_filter (str):
            A `monitoring
            filter <https://cloud.google.com/monitoring/api/v3/filters>`__
            specifying a ``TimeSeries`` aggregating values. Must have
            ``ValueType = DISTRIBUTION`` and ``MetricKind = DELTA`` or
            ``MetricKind = CUMULATIVE``.
        range_ (google.cloud.monitoring_v3.types.Range):
            Range of values considered "good." For a
            one-sided range, set one bound to an infinite
            value.
    """

    distribution_filter: str = proto.Field(
        proto.STRING,
        number=4,
    )
    range_: "Range" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="Range",
    )


class WindowsBasedSli(proto.Message):
    r"""A ``WindowsBasedSli`` defines ``good_service`` as the count of time
    windows for which the provided service was of good quality. Criteria
    for determining if service was good are embedded in the
    ``window_criterion``.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        good_bad_metric_filter (str):
            A `monitoring
            filter <https://cloud.google.com/monitoring/api/v3/filters>`__
            specifying a ``TimeSeries`` with ``ValueType = BOOL``. The
            window is good if any ``true`` values appear in the window.

            This field is a member of `oneof`_ ``window_criterion``.
        good_total_ratio_threshold (google.cloud.monitoring_v3.types.WindowsBasedSli.PerformanceThreshold):
            A window is good if its ``performance`` is high enough.

            This field is a member of `oneof`_ ``window_criterion``.
        metric_mean_in_range (google.cloud.monitoring_v3.types.WindowsBasedSli.MetricRange):
            A window is good if the metric's value is in
            a good range, averaged across returned streams.

            This field is a member of `oneof`_ ``window_criterion``.
        metric_sum_in_range (google.cloud.monitoring_v3.types.WindowsBasedSli.MetricRange):
            A window is good if the metric's value is in
            a good range, summed across returned streams.

            This field is a member of `oneof`_ ``window_criterion``.
        window_period (google.protobuf.duration_pb2.Duration):
            Duration over which window quality is evaluated. Must be an
            integer fraction of a day and at least ``60s``.
    """

    class PerformanceThreshold(proto.Message):
        r"""A ``PerformanceThreshold`` is used when each window is good when
        that window has a sufficiently high ``performance``.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            performance (google.cloud.monitoring_v3.types.RequestBasedSli):
                ``RequestBasedSli`` to evaluate to judge window quality.

                This field is a member of `oneof`_ ``type``.
            basic_sli_performance (google.cloud.monitoring_v3.types.BasicSli):
                ``BasicSli`` to evaluate to judge window quality.

                This field is a member of `oneof`_ ``type``.
            threshold (float):
                If window ``performance >= threshold``, the window is
                counted as good.
        """

        performance: "RequestBasedSli" = proto.Field(
            proto.MESSAGE,
            number=1,
            oneof="type",
            message="RequestBasedSli",
        )
        basic_sli_performance: "BasicSli" = proto.Field(
            proto.MESSAGE,
            number=3,
            oneof="type",
            message="BasicSli",
        )
        threshold: float = proto.Field(
            proto.DOUBLE,
            number=2,
        )

    class MetricRange(proto.Message):
        r"""A ``MetricRange`` is used when each window is good when the value x
        of a single ``TimeSeries`` satisfies
        ``range.min <= x <= range.max``. The provided ``TimeSeries`` must
        have ``ValueType = INT64`` or ``ValueType = DOUBLE`` and
        ``MetricKind = GAUGE``.

        Attributes:
            time_series (str):
                A `monitoring
                filter <https://cloud.google.com/monitoring/api/v3/filters>`__
                specifying the ``TimeSeries`` to use for evaluating window
                quality.
            range_ (google.cloud.monitoring_v3.types.Range):
                Range of values considered "good." For a
                one-sided range, set one bound to an infinite
                value.
        """

        time_series: str = proto.Field(
            proto.STRING,
            number=1,
        )
        range_: "Range" = proto.Field(
            proto.MESSAGE,
            number=4,
            message="Range",
        )

    good_bad_metric_filter: str = proto.Field(
        proto.STRING,
        number=5,
        oneof="window_criterion",
    )
    good_total_ratio_threshold: PerformanceThreshold = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="window_criterion",
        message=PerformanceThreshold,
    )
    metric_mean_in_range: MetricRange = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="window_criterion",
        message=MetricRange,
    )
    metric_sum_in_range: MetricRange = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="window_criterion",
        message=MetricRange,
    )
    window_period: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=4,
        message=duration_pb2.Duration,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
